using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing;
using Microsoft.VisualBasic.FileIO;


namespace AGifCreator
{
    /* Class  :FileExplorer
     * Author : Chandana Subasinghe
     * Date   : 10/03/2006
     * Discription : This class use to create the tree view and load 
     *               directories and files in to the tree
     *          
     */
    class FileTreeExplorer
    {
        public FileTreeExplorer()
        {
        }

        /* Method :CreateTree
         * Author : Chandana Subasinghe
         * Date   : 10/03/2006
         * Discription : This is use to creat and build the tree
         *          
         */

        public bool CreateTree(TreeView treeView)
        {
            bool returnValue = false;
                 
            try
            {
                // Create Desktop
                TreeNode desktop = new TreeNode();
                desktop.Text = "Desktop";
                desktop.Tag = "Desktop";
                desktop.Nodes.Add("");
                treeView.Nodes.Add(desktop);
                // Get driveInfo
                foreach (DriveInfo drv in DriveInfo.GetDrives())
                {
                    TreeNode fChild = new TreeNode();
                    if (drv.DriveType == DriveType.CDRom)
                    {
                        fChild.ImageIndex = 1;
                        fChild.SelectedImageIndex = 1;
                    }
                    else if (drv.DriveType == DriveType.Fixed)
                    {
                        fChild.ImageIndex = 0;
                        fChild.SelectedImageIndex = 0;
                    }
                    fChild.Text = drv.Name;
                    fChild.Nodes.Add("");
                    // treeView.Nodes.Add(fChild);
                    desktop.Nodes.Add(fChild);
                    returnValue = true;
                }
            }
            catch (Exception ex)
            {
                returnValue = false;
            }
            return returnValue;
            
        }

        /* Method :EnumerateDirectory
         * Author : Chandana Subasinghe
         * Date   : 10/03/2006
         * Discription : This is use to Enumerate directories and files
         *          
         */
        public TreeNode EnumerateDirectory(TreeNode parentNode)
        {
            try
            {
                DirectoryInfo rootDir;

                // To fill Desktop
                Char [] arr={'\\'};
                string [] nameList=parentNode.FullPath.Split(arr);
                string path = "";

                if (nameList.GetValue(0).ToString() == "Desktop")
                {
                    path = SpecialDirectories.Desktop+"\\";

                    for (int i = 1; i < nameList.Length; i++)
                    {
                        path = path + nameList[i] + "\\";
                    }

                    rootDir = new DirectoryInfo(path);
                }
             // for other Directories
                else
                {
                    rootDir = new DirectoryInfo(parentNode.FullPath + "\\");
                }
                
                parentNode.Nodes[0].Remove();
                foreach (DirectoryInfo dir in rootDir.GetDirectories())
                {
                    TreeNode node = new TreeNode();
                    node.Text = dir.Name;
                    node.Nodes.Add("");
                    parentNode.Nodes.Add(node);
                }
                //Fill files
                foreach (FileInfo file in rootDir.GetFiles())
                {
                    TreeNode node = new TreeNode();
                    node.Text = file.Name;
                    node.ImageIndex = 2;
                    node.SelectedImageIndex = 2;
                    parentNode.Nodes.Add(node);
                }
            }

            catch (Exception ex)
            {
                //TODO : 
            }
           
            return parentNode;
        }

        public TreeNode ExpandNode(TreeView treeView, DirectoryInfo dirInfo)
        {
            if (dirInfo == null)
                return treeView.TopNode;

            TreeNode parentNode = ExpandNode(treeView, dirInfo.Parent);
            TreeNode node = FindNode(parentNode.Nodes, dirInfo.Name);
            if (node != null)
                node.Expand();

            return node;
        }

        public void DumpTree(TreeNodeCollection nodes, int level)
        {
            if (nodes == null)
                return;

            int cnt = nodes.Count;
            for (int idx = 0; idx < cnt; idx++)
            {
                System.Diagnostics.Debug.WriteLine(
                    level.ToString() +
                    " Tag[" + idx.ToString() + "]=" + nodes[idx].Tag
                    + " Name=" + nodes[idx].Name
                    + " Text=" + nodes[idx].Text
                    + " Cnt=" + nodes[idx].Nodes.Count.ToString());

                DumpTree(nodes[idx].Nodes, level + 1);
            }
        }

        public TreeNode FindNode(TreeNodeCollection nodes, string text)
        {
            if (nodes == null)
                return null;

            foreach (TreeNode child in nodes)
            {
                if (string.Compare(child.Text, text, true) == 0)
                    return child;
            }

            TreeNode found = null;
            for (int idx = 0; idx < nodes.Count && found == null; idx++)
            {
                found = FindNode(nodes[idx].Nodes, text);
            }

            return found;
        }
    }
}
