﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;


namespace AGifCreator
{
    /// <summary>
    /// Click on directory path sub-directory to prune back to that level.
    /// Ex:     c:/foo/bar/car
    /// 
    /// Click on bar will cause c:/foo/bar to be the base directory.
    /// 
    /// Author: Dennis Lang  2009       
    /// http://home.comcast.net/~lang.dennis/
    /// 
    /// </summary>
    public partial class FilePathBar : UserControl
    {
        public FilePathBar()
        {
            InitializeComponent();
            this.pathBar.SelectionFont = new Font(this.pathBar.Font.FontFamily, this.pathBar.Font.Size, FontStyle.Bold);
        }

        public void LoadFromRegistry(Microsoft.Win32.RegistryKey key)
        {
        }

        public void SaveToRegistry(Microsoft.Win32.RegistryKey key)
        {
        }

        public string PathAtPosition(Point position)
        {
            int pos = this.pathBar.GetCharIndexFromPosition(position);

            string path = this.pathBar.Text;
            DirectoryInfo dirInfo = new DirectoryInfo(path);

            while (dirInfo != null && dirInfo.FullName.Length > pos)
            {
                path = dirInfo.FullName;
                dirInfo = dirInfo.Parent;
            }

            return path;
        }

        private void pathBar_MouseDown(object sender, MouseEventArgs e)
        {
            string path = PathAtPosition(e.Location);
            if (path != this.pathBar.Text)
            {
                this.pathBar.Text = path;
                if (this.Changed != null)
                    this.Changed(this, EventArgs.Empty);
            }
        }

        private void pathBar_MouseMove(object sender, MouseEventArgs e)
        {
            this.pathBar.Focus();
            if (this.pathBar.Text.Length > 0)
            {
                DirectoryInfo dirInfo = new DirectoryInfo(PathAtPosition(e.Location));
                this.pathBar.SelectionStart = dirInfo.FullName.Length - dirInfo.Name.Length;
                this.pathBar.SelectionLength = dirInfo.Name.Length;
            }
        }

        public string FilePath
        {
            get { return this.pathBar.Text; }
            set { this.pathBar.Text = value; }
        }

        public event EventHandler Changed;


    }
}
