using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing;
using Microsoft.VisualBasic.FileIO;


namespace AGifCreator
{
    ///
    /// Author : Chandana Subasinghe
    /// Date   : 10/03/2006
    /// 
    /// Populate tree with file system (drives, directories and files)
    /// 
    /// Modified by: Dennis Lang  2009       
    /// http://home.comcast.net/~lang.dennis/
    /// 
    class FileTreeExplorer
    {
        public FileTreeExplorer()
        {
        }

        /// <summary>
        /// Populate first level of treeView with disk drives.
        /// </summary>
        public bool CreateTree(TreeView treeView)
        {
            bool returnValue = false;
                 
            try
            {
                // Create Desktop
                TreeNode desktop = new TreeNode();
                desktop.Text = "Desktop";
                desktop.Tag = "Desktop";
                desktop.Nodes.Add("");
                treeView.Nodes.Add(desktop);

                // Get driveInfo
                foreach (DriveInfo drv in DriveInfo.GetDrives())
                {
                    TreeNode fChild = new TreeNode();
                    if (drv.DriveType == DriveType.CDRom)
                    {
                        fChild.ImageIndex = 1;
                        fChild.SelectedImageIndex = 1;
                    }
                    else if (drv.DriveType == DriveType.Fixed)
                    {
                        fChild.ImageIndex = 0;
                        fChild.SelectedImageIndex = 0;
                    }
                    fChild.Text = drv.Name;
                    fChild.Nodes.Add("");       // assume children are possible
                    desktop.Nodes.Add(fChild);
                    returnValue = true;
                }
            }
            catch (Exception)
            {
                returnValue = false;
            }
            return returnValue;
            
        }

        /// <summary>
        /// Populate children with directories and files.
        /// </summary>
        public TreeNode EnumerateDirectory(TreeNode parentNode)
        {
            try
            {
                if (parentNode.Text == "Desktop")
                    return parentNode;

                DirectoryInfo rootDir = 
                        new DirectoryInfo(parentNode.FullPath.Substring(8));
                
                parentNode.Nodes[0].Remove();
                foreach (DirectoryInfo dir in rootDir.GetDirectories())
                {
                    TreeNode node = new TreeNode();
                    node.Text = dir.Name;
                    node.Nodes.Add("");     // assume children are possible.
                    parentNode.Nodes.Add(node);
                }

                //Fill files
                foreach (FileInfo file in rootDir.GetFiles())
                {
                    TreeNode node = new TreeNode();
                    node.Text = file.Name;
                    node.ImageIndex = 2;
                    node.SelectedImageIndex = 2;
                    parentNode.Nodes.Add(node);
                }
            }

            catch (Exception)
            {
                //TODO : 
            }
           
            return parentNode;
        }

        public TreeNode ExpandNode(TreeView treeView, DirectoryInfo dirInfo)
        {
            if (dirInfo == null)
            {
                treeView.Nodes[0].Expand();
                return treeView.Nodes[0];
            }

            TreeNode parentNode = ExpandNode(treeView, dirInfo.Parent);
            TreeNode node = FindNode(parentNode.Nodes, dirInfo.Name);
            if (node != null)
            {
                node.Expand();
                treeView.TopNode = node;  // make node visible.
            }

            return node;
        }

        /// <summary>
        /// Scan text for case insensitive match.
        /// </summary>
        /// <param name="nodes">Start search at</param>
        /// <param name="text">look for match with</param>
        /// <returns>matching node or null</returns>
        public TreeNode FindNode(TreeNodeCollection nodes, string text)
        {
            if (nodes == null)
                return null;

            foreach (TreeNode child in nodes)
            {
                if (string.Compare(child.Text, text, true) == 0)
                    return child;
            }

            TreeNode found = null;
            for (int idx = 0; idx < nodes.Count && found == null; idx++)
                found = FindNode(nodes[idx].Nodes, text);

            return found;
        }
    }
}
