﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace AGifCreator
{
    /// <summary>
    /// Movie (VCR) controls to play a sequence of image frames.
    /// See FramePlayer for full implementation.
    /// 
    /// Author: Dennis Lang  2009       
    /// http://home.comcast.net/~lang.dennis/
    /// </summary>    
    public partial class FrameControl : UserControl
    {
        public class EventFrame : EventArgs
        {
            public EventFrame(int frame, eAction action)
            {
                Frame = frame;
                Action = action;
            }
            public enum eAction { eNone, eSet, ePlayF, ePlayR, eStop, eCycleF, eCycleR };
            public int Frame;
            public eAction Action;
        };

        public FrameControl()
        {
            InitializeComponent();
            this.Slider = this.sliderMin;
            this.timer.Tick += new EventHandler(timer_Tick);
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            switch (this.action)
            {
                case EventFrame.eAction.eNone:
                    break;
                case EventFrame.eAction.eCycleF:
                    Slider = sliderValue < sliderMax ? (sliderValue + 1) : sliderMin;
                    break;
                case EventFrame.eAction.ePlayF:
                    if (this.Slider < this.sliderMax)
                        Slider = Slider + 1;
                    else
                        this.action = EventFrame.eAction.eNone;
                    break;
                case EventFrame.eAction.ePlayR:
                    if (this.Slider > this.sliderMin)
                        Slider = Slider - 1;
                    else
                        this.action = EventFrame.eAction.eNone;
                    break;
                case EventFrame.eAction.eSet:
                    break;
            }

            if (this.action == EventFrame.eAction.eNone)
            {
                stopBtn_Click(sender, e);
            }
            else if (this.Changed != null)
                this.Changed(this, new EventFrame(this.sliderValue, this.action));
        }

        private void revStepBtn_Click(object sender, EventArgs e)
        {
            this.action = EventFrame.eAction.eSet;
            this.sPlayer.Stream = Properties.Resources.click1;
            this.sPlayer.Play();

            if (this.sliderValue > this.sliderMin)
                Slider = this.sliderValue - 1;
            else
                Slider = this.sliderMax;

            if (this.Changed != null)
                this.Changed(this, new EventFrame(this.sliderValue, EventFrame.eAction.eSet));
        }

        private void revBtn_Click(object sender, EventArgs e)
        {
            if (this.action == EventFrame.eAction.ePlayR)
            {
                stopBtn_Click(sender, e);
                return;
            }

            if (this.sliderValue <= this.sliderMin)
                this.Slider = this.sliderMax;

            this.action = EventFrame.eAction.ePlayR;
            // this.sPlayer.Stream = Properties.Resources.drum1;
            // this.sPlayer.Play();
            this.timer.Start();

            if (this.Changed != null)
                this.Changed(this, new EventFrame(this.sliderValue, EventFrame.eAction.ePlayR));
        }

        private void stopBtn_Click(object sender, EventArgs e)
        {
            this.action = EventFrame.eAction.eStop;
            this.sPlayer.Stream = Properties.Resources.drum1;
            this.sPlayer.Play();
            this.timer.Stop();

            if (this.Changed != null)
                this.Changed(this, new EventFrame(this.sliderValue, EventFrame.eAction.eStop));
        }

        private void cycleBtn_Click(object sender, EventArgs e)
        {
            if (this.timer.Enabled)
            {
                stopBtn_Click(sender, e);
                return;
            }

            if (this.sliderValue >= this.sliderMax)
                this.Slider = this.sliderMin;

            this.action = EventFrame.eAction.eCycleF;
            this.sPlayer.Stream = Properties.Resources.online1;
            this.sPlayer.Play();
            this.timer.Start();

            if (this.Changed != null)
                this.Changed(this, new EventFrame(this.sliderValue, EventFrame.eAction.eCycleF));
        }

        private void playBtn_Click(object sender, EventArgs e)
        {
            if (this.action == EventFrame.eAction.ePlayF)
            {
                stopBtn_Click(sender, e);
                return;
            }

            if (this.sliderValue >= this.sliderMax)
                this.Slider = this.sliderMin;

            this.action = EventFrame.eAction.ePlayF;
            this.sPlayer.Stream = Properties.Resources.online1;
            this.sPlayer.Play();
            this.timer.Start();

            if (this.Changed != null)
                this.Changed(this, new EventFrame(this.sliderValue, EventFrame.eAction.ePlayF));
        }

        private void playStepBtn_Click(object sender, EventArgs e)
        {
            this.action = EventFrame.eAction.eSet;
            this.sPlayer.Stream = Properties.Resources.click1;
            this.sPlayer.Play();

            if (this.sliderValue < this.sliderMax)
                Slider = this.sliderValue + 1;
            else
                Slider = this.sliderMin;

            if (this.Changed != null)
                this.Changed(this, new EventFrame(this.sliderValue, EventFrame.eAction.eSet));
        }

        private void frameBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                try
                {
                    Slider = Int32.Parse(this.frameBox.Text);

                    if (this.Changed != null)
                        this.Changed(this, new EventFrame(this.sliderValue, EventFrame.eAction.eSet));
                }
                catch
                {
                    this.frameBox.Text = this.sliderValue.ToString();
                }
            }
        }

        private void timeLine_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                float per = (float)e.Location.X / this.timeLine.Width;
                int v = this.sliderMin + (int)(per * (this.sliderMax - this.sliderMin + 1));
                Slider = v;

                if (this.Changed != null)
                    this.Changed(this, new EventFrame(this.sliderValue, EventFrame.eAction.eSet));
            }
        }

        public int Slider
        {
            get { return this.sliderValue; }
            set 
            {
                this.frameBox.Text = value.ToString();

                this.sliderValue = value; 
                Point newLocation = this.sliderBtn.Location;
                int sliderRange = this.sliderMax - this.sliderMin;
                float sliderPer = (float)(this.sliderValue - this.sliderMin) / sliderRange;

                int newX = (int)((this.timeLine.Width - this.sliderBtn.Width) * sliderPer);
                newLocation.X = newX;
                this.sliderBtn.Location = newLocation;
            }
        }

        public int SliderMin
        {
            get { return this.sliderMin; }
            set { this.sliderMin = value; BuildTimeLine();  }
        }
        public int SliderMax
        {
            get { return this.sliderMax; }
            set { this.sliderMax = value; BuildTimeLine(); }
        }

        private void BuildTimeLine()
        {
            this.timeLine.BackgroundImage = global::AGifCreator.Properties.Resources.blueH16W50;
            int numFrames = this.sliderMax - this.sliderMin + 1;
            if (numFrames > 1 && numFrames < 40)
            {
                Graphics g = Graphics.FromImage(this.timeLine.BackgroundImage);
                int height = this.timeLine.BackgroundImage.Height - 1;
                for (int i = 1; i < numFrames; i++)
                {
                    int xPos = this.timeLine.BackgroundImage.Width * i / numFrames;
                    Point p1 = new Point(xPos, 0);
                    Point p2 = new Point(xPos, height);
                    g.DrawLine(Pens.Blue, p1, p2);
                }
                g.Flush();
            }
        }

        private int sliderMin = 0;
        private int sliderValue = 0;
        private int sliderMax = 100;
        private EventFrame.eAction action = EventFrame.eAction.eNone;
        private System.Media.SoundPlayer sPlayer = new System.Media.SoundPlayer();
        public event EventHandler Changed;
    }
}
