﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;


namespace AGifCreator
{
    /// <summary>
    /// Frame player which allows user to control the playing of 
    /// a sequence of image frames.
    /// Uses FrameControl to sequence frames.
    /// Images provided by outside source.
    /// 
    /// Author: Dennis Lang  2009       
    /// http://home.comcast.net/~lang.dennis/
    /// 
    /// </summary>
    public partial class FramePlayer : UserControl
    {
        public FramePlayer()
        {
            InitializeComponent();
        }

        private void ViewBtn_Click(object sender, EventArgs e)
        {
            ToolStripDropDownItem viewItem = (ToolStripDropDownItem)sender;
            ToolStripDropDownMenu viewItems = viewItem.DropDown.OwnerItem.Owner as ToolStripDropDownMenu;

            foreach (ToolStripDropDownItem item in viewItems.Items)
                item.Image = null;

            viewItem.Image = global::AGifCreator.Properties.Resources.greenCheck32;
            viewItems.OwnerItem.Text = viewItem.Text + "..."; ;
            this.pictureBox.SizeMode = (PictureBoxSizeMode)viewItem.Tag;
        }

        public Image ViewImage
        {
            get { return this.pictureBox.Image; }
            set { this.pictureBox.Image = value; }
        }

        public int ViewFrame
        {
            get { return this.frameControl.Slider; }
            set { this.frameControl.Slider = value; }
        }

        public int ViewFrameMax
        {
            get { return this.frameControl.SliderMax; }
            set { this.frameControl.SliderMax = value; }
        }

        private void frameControl_Changed(object sender, EventArgs e)
        {
            FrameControl.EventFrame evtFrame = e as FrameControl.EventFrame;

            switch (evtFrame.Action)
            {
                case FrameControl.EventFrame.eAction.eNone:
                    this.statusBar.Text = "";
                    break;
                case FrameControl.EventFrame.eAction.ePlayF:
                    this.statusBar.Text = "PlayForward";
                    break;
                case FrameControl.EventFrame.eAction.ePlayR:
                    this.statusBar.Text = "PlayReverse";
                    break;
                case FrameControl.EventFrame.eAction.eCycleF:
                    this.statusBar.Text = "CycleForward";
                    break;
                case FrameControl.EventFrame.eAction.eSet:
                    this.statusBar.Text = "Set Frame";
                    break;
                case FrameControl.EventFrame.eAction.eStop:
                    this.statusBar.Text = "Stop";
                    break;
            }

            if (this.Changed != null)
                this.Changed(sender, e);
        }


        /// <summary>
        /// Add points together, return sum.
        /// </summary>
        private Point Add(Point p1, Point p2)
        {
            return new Point(p1.X + p2.X, p1.Y + p2.Y);
        }

        /// <summary>
        /// Prompt for new maximum frame count.
        /// </summary>
        private void customSpeedBtn_Click(object sender, EventArgs e)
        {
            PromptValue promptValue = new PromptValue("CustomSpeed", "Msec:", this.speedMsec);
            promptValue.Location = Add(this.PointToScreen(this.Location), new Point(50, 50));
            promptValue.StartPosition = System.Windows.Forms.FormStartPosition.Manual;

            if (promptValue.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    this.speedMsec = ushort.Parse(promptValue.textBox.Text);
                }
                catch { }
            }
        }


        public ushort speedMsec = 800;
        public event EventHandler Changed;

    }


    #region Prompt for value.
    /// <summary>
    /// Simple dialog to collect frame number
    /// </summary>
    public class PromptValue : Form
    {
        public PromptValue(string titleStr, string labelStr, int value)
        {
            textBox = new TextBox();

            Label lbl = new Label();
            Button okBtn = new Button();
            Button cancelBtn = new Button();

            this.Text = titleStr;
            textBox.Text = value.ToString();
            lbl.Text = labelStr;
            okBtn.Text = "Accept";
            cancelBtn.Text = "Cancel";

            this.Size = new Size(200, 100);
            lbl.Location = new Point(5, 5);
            textBox.Location = new Point(5+lbl.Width+5, 5);
            okBtn.Location = new Point(5, 30);
            cancelBtn.Location = new Point(100, 30);

            this.SuspendLayout();
            this.Controls.Add(lbl);
            this.Controls.Add(textBox);
            this.Controls.Add(okBtn);
            this.Controls.Add(cancelBtn);
            this.ResumeLayout();

            okBtn.Click += new EventHandler(okBtn_Click);
            cancelBtn.Click += new EventHandler(cancelBtn_Click);
            this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.mouseMove);
        }

        public TextBox textBox;

        public void okBtn_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
            Close();
        }
        public void cancelBtn_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            Close();
        }

        private Point mouseStartAt = Point.Empty;
        private void mouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                if (mouseStartAt == Point.Empty)
                {
                    mouseStartAt = e.Location;
                    this.textBox.BackColor = Color.Aqua;
                }
                else
                {
                    int off = Math.Abs(e.X - mouseStartAt.X) > Math.Abs(e.Y - mouseStartAt.Y) ?
                        (e.X - mouseStartAt.X) : (e.Y - mouseStartAt.Y);
                    this.textBox.Text = Math.Max(0, Int32.Parse(textBox.Text) + off/10).ToString();
                }
            }
            else
            {
                mouseStartAt = Point.Empty;
                this.textBox.BackColor = Color.White;
            }
        }
    }

    #endregion
}
